// @Name: ai_continuous.cpp
// @Author: Smacq
// @Copyright: Smacq
// @Date: 2025/02/08
// @Brief:
// This code focuses on the Ai functions of the USB - 5000 device, performing continuous acquisition.
// It acquires 100 data points from the Ai0 channel 10 times.
// You can refer to the user manual for more information about functions or device details.
//
// Execution result when only connecting USB for USB - 5000:
// Device_number: 1
// 0
// Device_Index: 0 Device_SN: 66666666 Device_Model: 5722
// 0
// ...
// 0
// remain_buffer: 2
// Ai0 100 data points
//  [-0.09876205 -0.09876205 -0.09876205 -0.09876205 -0.09907459 -0.09907459
//...
//  -0.09876205 -0.09844952 -0.09844952 -0.09876205]
// remain_buffer: 3
// Ai0 100 data points
//  [ 0.         -0.00031254  0.          0.         -0.00093761 -0.00031254
// ...
//  -0.09813698 -0.09782444 -0.09782444 -0.0975119 ]
// 0
// 0
// 0

#include <QCoreApplication>
#include <QDebug>
extern "C"
{
#include "USB5000.h"
}

int main(int argc, char *argv[])
{
    QCoreApplication a(argc, argv);
    // Initialize Ai_data_array
    // Usually the array size need >= acquisition data points size.
    // Data type is float32, it is not modifiable.
    // The data points saved to Ai_data_array arrange from opened Ai channel
    float Ai_data_array[2400];

    // Device index starts from zero
    int DevIndex = 0;

    // Ai voltage range:
    // 10 = -10V to +10V and so on
    // Voltage range param support 10/5
    float Range = 10.0f;

    // Ai channel sample mode:
    // 0 = Continuous acquisition or 1 = Oneshot, param support 0/1
    unsigned char AiSampleMode = 0;

    // Acquisition amount of data:
    // Each opened Ai channel will get this amount of data.
    // Suggest value within sampling rate 10%~20%.
    unsigned long Num = 100;

    // Configuration values for Ai channels:
    // Device model channel number start from 0
    unsigned char Channel = 0;

    // Set channel ON/OFF
    unsigned char Channel_ON = 1;
    unsigned char Channel_OFF = 0;

    // Ai Sampling period, unit ns, steps 10ns:
    // USB - 3000 is asynchronous DAQ, single channel max rate is 1M = 1000.
    // Multichannel each channel share out samplerate, 1M/channel.
    // sample rate and period:
    // 100k = 10000
    // 10k = 100000
    // 1k = 1000000
    // 100 = 10000000
    // 10 = 100000000
    // 1 = 1000000000
    unsigned int AiSamplePeriod = 1000000;

    // Ai acquisition trig source:
    // 0 = SetUSB5AiSoftTrig(), 1 = Ext_Trig_In, 2~5 = Din_0~3,
    // 7 = with DI, 8 = with DO, 9~12 = with AO~3, 255 = GlobalSoftTrig
    // Supported parameters: 0/1/2/3/4/5/7/8/9/10/11/12/255
    char AiTrigSource = 0;

    // Ai acquisition clock source:
    // 0 = SetUSB5AiSampleRate(), 1 = Conv_In, 3 = Di, 4 = DO, 5~8 = AO~3
    // Supported parameters: 0/1/3/4/5/6/7/8
    char AiConvSource = 0;

    // Timeout unit ms:
    // Suggest Timeout > (Num*Channel_count/Samplerate)*1000
    // Avoid USB5GetAi() func timeout, if timeout return -7.
    long Timeout = 200;

    int error_code;
    // Find connected DAQ number
    int Device_number = FindUSB5DAQ();
    printf("Device_number: %d\n", Device_number);

    // Open specific device and print return value.
    error_code = USB5OpenDevice(DevIndex);
    printf("%d\n", error_code);

    // Get and print Device_SN
    unsigned long Device_SN = USB5ReadDevcieSN(DevIndex);
    unsigned long Device_Model = USB5ReadDevcieModel(DevIndex);
    printf("Device_Index: %d Device_SN: %ld Device_Model: %ld\n", DevIndex, Device_SN, Device_Model);

    // Turn OFF Ai channel 0 - 8 (default Ai channel 0 - 8 is ON)
    for (int channel = 0; channel < 8; ++channel)
    {
        error_code = SetUSB5AiChanSel(DevIndex, channel, Channel_OFF);
        if (error_code != 0)
        {
            printf("%d\n", error_code);
        }
    }

    // Turn ON and set input voltage Range for Ai channel 0
    for (int channel = 0; channel < 1; ++channel)
    {
        error_code = SetUSB5AiChanSel(DevIndex, channel, Channel_ON);
        printf("%d\n", error_code);
        error_code = SetUSB5AiRange(DevIndex, channel, Range);
        printf("%d\n", error_code);
    }

    // Set Device Ai sample mode
    error_code = SetUSB5AiSampleMode(DevIndex, AiSampleMode);
    printf("%d\n", error_code);

    // Set Device sample rate
    error_code = SetUSB5AiSampleRate(DevIndex, AiSamplePeriod);
    printf("%d\n", error_code);

    // Set Device clock source
    error_code = SetUSB5AiConvSource(DevIndex, AiConvSource);
    printf("%d\n", error_code);

    // Clear AI FIFO
    error_code = SetUSB5ClrAiFifo(DevIndex);
    printf("%d\n", error_code);

    // Set Device trig source
    error_code = SetUSB5AiTrigSource(DevIndex, AiTrigSource);
    printf("%d\n", error_code);

    // Enable Ai software trig acquisition
    error_code = SetUSB5AiSoftTrig(DevIndex);
    printf("%d\n", error_code);

    // Continuous acquisition execution 10 times GetAiChans()
    for (int i = 0; i < 10; ++i)
    {
        int remain_buffer = USB5GetAi(DevIndex, Num, Ai_data_array, Timeout);
        /*
        remain_buffer range 0~2000000
        USB5GetAi() return the remaining Device FIFO buffer value.
        FIFO buffer 100 data points save to Ai_data_array.
        Channel_0 100 data points is Ai_data_array[0:100]
        If open channel 0~1, Channel_1 100 data points is Ai_data_array[100:200] and so on.
        If remain_buffer increase you should run USB5GetAi() more quick, let remain_buffer steady.
        */
        printf("remain_buffer: %d\n", remain_buffer);
        printf("Ai0 100 data points\n");
        for (int j = 0; j < 100; ++j)
        {
            printf("%f ", Ai_data_array[j]);
        }
        printf("\n");
        // Add your code here can handle data
    }

    // Disable Ai software trig acquisition, clear flag
    error_code = SetUSB5ClrAiTrigger(DevIndex);
    printf("%d\n", error_code);

    // Clear AI FIFO
    error_code = SetUSB5ClrAiFifo(DevIndex);
    printf("%d\n", error_code);

    // Close Device
    error_code = USB5CloseDevice(DevIndex);
    printf("%d\n", error_code);

    return 0;
}
