// @Name: ao_waveform.cpp
// @Author: Smacq
// @Copyright: Smacq
// @Date: 2025/02/08
// @Brief:
// This code focuses on the Ao functions of the USB - 5000 device, performing waveform output.
// It outputs a -1V ~ 1V sine wave on Ao channel 0, with 1000 data points and a 10 - second output duration.
// The device's AO analog voltage range is -10V ~ +10V.
// The Ao channels vary depending on the module, either Ao_4 or NONE.
// You can refer to the user manual for more information about functions or device details.
//
// Execution result when only connecting USB for USB - 5100(Ao_4):
// 0
// ...
// 0
// Ao outputting sine wave.
// Ao output ends.
// 0
// 0
// 0

#include <QCoreApplication>
#include <QDebug>
#include <QThread>
extern "C"
{
#include "USB5000.h"
}
#include <math.h>
int main(int argc, char *argv[])
{
    QCoreApplication a(argc, argv);

    // init -1 ~ 1V sine wave to Ao_data_array
    float Ao_data_array[1000];

    // Device index starts from zero
    int DevIndex = 0;

    // Ao channel, Ao channel 0 to Ao channel 3
    unsigned char AOChannel_0 = 0;

    // Ao channel sample mode:
    // 0 = continuous non - loop output to computer buffer
    // 1 = FIFO output loop determined by SetUSB5AoCycle()
    unsigned char AoSampleMode = 1;

    // Ao Sampling period, unit ns, steps 10ns:
    // USB - 3000 is synchronous Ao, max rate is 100k = 10000.
    // sample rate and period:
    // 10k = 100000
    // 1k = 1000000
    // 100 = 10000000
    // 10 = 100000000
    // 1 = 1000000000
    unsigned int AoSampleRate = 10000000;

    // Ao acquisition trig source:
    // 0 = SetUSB5AoSoftTrig(), 1 = Ext_Trig_In, 2~5 = Din_0~3,
    // 6 = with Ai, 7 = with Di, 7 = with Do, 9~12 = with AO~3, 255 = GlobalSoftTrig
    // Supported parameters: 0/1/2/3/4/5/6/7/8/9/10/11/12/255
    char AoTrigSource = 0;

    // Ao acquisition clock source:
    // 0 = SetUSB5AoSampleRate(), 1 = Conv_In, 2 = Ai, 3 = Di, 5~8 = AO~3
    // Supported parameters: 0/1/2/3/5/6/7/8
    char AoConvSource = 0;

    // Set waveform output cycle
    // 0 = infinite loop output
    // 1 = 1 time, 2 = 2 times and so on.
    // Supported parameters: 0 ~ 4294967295
    unsigned int AoCycle = 1;

    // Download to Ao FIFO data number
    unsigned int Num = 1000;
    int error_code;

    for (int i = 0; i < 1000; ++i)
    {
        double x = -2 * 3.14 + (4 * 3.14 * i) / 999;
        Ao_data_array[i] = (float)sin(x);
    }

    // Open specific device and print return value.
    error_code = USB5OpenDevice(DevIndex);
    printf("%d\n", error_code);

    // Disable Ao software trig, clear flag
    error_code = SetUSB5ClrAoTrigger(DevIndex, AOChannel_0);
    printf("%d\n", error_code);

    // Clear FIFO
    error_code = SetUSB5ClrAoFifo(DevIndex, AOChannel_0);
    printf("%d\n", error_code);

    // Set Device Ao sample mode
    error_code = SetUSB5AoSampleMode(DevIndex, AOChannel_0, AoSampleMode);
    printf("%d\n", error_code);

    // Set Device sample rate
    error_code = SetUSB5AoSampleRate(DevIndex, AOChannel_0, AoSampleRate);
    printf("%d\n", error_code);

    // Set Device trig source
    error_code = SetUSB5AoTrigSource(DevIndex, AOChannel_0, AoTrigSource);
    printf("%d\n", error_code);

    // Set Device clock source
    error_code = SetUSB5AoConvSource(DevIndex, AOChannel_0, AoConvSource);
    printf("%d\n", error_code);

    // Set digital output cycle 2 times
    error_code = SetUSB5AoCycle(DevIndex, AOChannel_0, AoCycle);
    printf("%d\n", error_code);

    // Download array data to Ao FIFO
    error_code = SetUSB5AoDataFifo(DevIndex, AOChannel_0, Ao_data_array, Num);
    printf("%d\n", error_code);

    // Enable Ao software trig start Ao output wave
    error_code = SetUSB5AoSoftTrig(DevIndex, AOChannel_0);
    printf("%d\n", error_code);

    // Necessary delay for Do wave output
    printf("Ao outputting sine wave.\n");
    QThread::sleep(10);
    printf("Ao outputting to ends\n");

    // Disable Ao software trig, clear flag
    error_code = SetUSB5ClrAoTrigger(DevIndex, AOChannel_0);
    printf("%d\n", error_code);

    // Clear FIFO
    error_code = SetUSB5ClrAoFifo(DevIndex, AOChannel_0);
    printf("%d\n", error_code);

    // Close Device and stop Ao wave output
    error_code = USB5CloseDevice(DevIndex);
    printf("%d\n", error_code);

    return 0;
}
