// @Name: ao_waveform_pc_infinite.cpp
// @Author: Smacq
// @Copyright: Smacq
// @Date: 2025/02/08
// @Brief:
// This code focuses on the Ao functions of the USB - 5000 device, performing waveform output.
// It outputs a -1V ~ 1V sine wave and a -5V ~ +5V square wave on Ao channel 0.
// The device's AO analog voltage range is -10V ~ +10V.
// The Ao channels vary depending on the module, either Ao_4 or NONE.
// You can refer to the user manual for more information about functions or device details.
//
// Execution result when only connecting USB for USB - 5100(Ao_4):
// 0
// ...
// 0
// Ao0 outputting -1V and 1V sine wave, Please Enter to next wave.
// 0
// 0
// 0
// 0
// Ao0 outputting -1V and 1V square wave, Please Enter to stop output.
// 0 AOChannel_0 is 0V
// 0
// 0
// 0

#include <QCoreApplication>
#include <QDebug>
extern "C"
{
#include "USB5000.h"
}
#include <math.h>
#include <QThread>
int main(int argc, char *argv[])
{
    QCoreApplication a(argc, argv);

    // init -1 ~ 1V sine wave to Ao_data_array
    float Ao_data_array_sine[1000];
    for (int i = 0; i < 1000; ++i)
    {
        double x = -2 * 3.14 + (4 * 3.14 * i) / 999;
        Ao_data_array_sine[i] = (float)sin(x);
    }
    // init -1V and 1V square wave 200 data point to Ao_data_array
    float Ao_data_array_square[200];
    for (int i = 0; i < 100; ++i)
    {
        Ao_data_array_square[i] = 1.0f;
    }
    for (int i = 100; i < 200; ++i)
    {
        Ao_data_array_square[i] = -1.0f;
    }

    // Device index starts from zero
    int DevIndex = 0;

    // Ao channel, Ao channel 0 to Ao channel 3
    unsigned char AOChannel_0 = 0;
    unsigned char AOChannel_1 = 1;
    unsigned char AOChannel_2 = 2;
    unsigned char AOChannel_3 = 3;

    // voltage value, range is -10 to +10
    float voltage_0 = 0.0f;

    // Ao channel sample mode:
    // 0 = continuous non - loop output to computer buffer
    // 1 = FIFO output loop determined by SetUSB5AoCycle()
    unsigned char AoSampleMode = 1;

    // Ao Sampling period, unit ns, steps 10ns:
    // USB - 3000 is synchronous Ao, max rate is 100k = 10000.
    // sample rate and period:
    // 10k = 100000
    // 1k = 1000000
    // 100 = 10000000
    // 10 = 100000000
    // 1 = 1000000000
    unsigned int AoSampleRate = 10000000;

    // Ao acquisition trig source:
    // 0 = SetUSB5AoSoftTrig(), 1 = Ext_Trig_In, 2~5 = Din_0~3,
    // 6 = with Ai, 7 = with Di, 7 = with Do, 9~12 = with AO~3, 255 = GlobalSoftTrig
    // Supported parameters: 0/1/2/3/4/5/6/7/8/9/10/11/12/255
    char AoTrigSource = 0;

    // Ao acquisition clock source:
    // 0 = SetUSB5AoSampleRate(), 1 = Conv_In, 2 = Ai, 3 = Di, 5~8 = AO~3
    // Supported parameters: 0/1/2/3/5/6/7/8
    char AoConvSource = 0;

    // Set waveform output cycle
    // 0 = infinite loop output
    // 1 = 1 time, 2 = 2 times and so on.
    // Supported parameters: 0 ~ 4294967295
    unsigned int AoCycle = 0;

    // Download to Ao FIFO data number for sine wave
    unsigned int Num_sine = 1000;
    // Download to Ao FIFO data number for square wave
    unsigned int Num_square = 200;

    int error_code;

    // Open specific device and print return value.
    error_code = USB5OpenDevice(DevIndex);
    printf("%d\n", error_code);

    // Disable Ao software trig, clear flag
    error_code = SetUSB5ClrAoTrigger(DevIndex, AOChannel_0);
    printf("%d\n", error_code);

    // Clear FIFO
    error_code = SetUSB5ClrAoFifo(DevIndex, AOChannel_0);
    printf("%d\n", error_code);

    // Set Device Ao sample mode
    error_code = SetUSB5AoSampleMode(DevIndex, AOChannel_0, AoSampleMode);
    printf("%d\n", error_code);

    // Set Device sample rate
    error_code = SetUSB5AoSampleRate(DevIndex, AOChannel_0, AoSampleRate);
    printf("%d\n", error_code);

    // Set Device trig source
    error_code = SetUSB5AoTrigSource(DevIndex, AOChannel_0, AoTrigSource);
    printf("%d\n", error_code);

    // Set Device clock source
    error_code = SetUSB5AoConvSource(DevIndex, AOChannel_0, AoConvSource);
    printf("%d\n", error_code);

    // Set digital output cycle 2 times
    error_code = SetUSB5AoCycle(DevIndex, AOChannel_0, AoCycle);
    printf("%d\n", error_code);

    // Download array data to Ao FIFO for sine wave
    error_code = SetUSB5AoDataFifo(DevIndex, AOChannel_0, Ao_data_array_sine, Num_sine);
    printf("%d\n", error_code);

    // Enable Ao software trig start Ao output wave
    error_code = SetUSB5AoSoftTrig(DevIndex, AOChannel_0);
    printf("%d\n", error_code);

    printf("Ao0 outputting -1V and 1V sine wave, Please Enter to next wave.\n");
    getchar();

    // Disable Ao software trig, clear flag
    error_code = SetUSB5ClrAoTrigger(DevIndex, AOChannel_0);
    printf("%d\n", error_code);

    // Clear FIFO
    error_code = SetUSB5ClrAoFifo(DevIndex, AOChannel_0);
    printf("%d\n", error_code);

    // Download array data to Ao FIFO for square wave
    error_code = SetUSB5AoDataFifo(DevIndex, AOChannel_0, Ao_data_array_square, Num_square);
    printf("%d\n", error_code);

    // Enable Ao software trig start Ao output wave
    error_code = SetUSB5AoSoftTrig(DevIndex, AOChannel_0);
    printf("%d\n", error_code);

    printf("Ao0 outputting -1V and 1V square wave, Please Enter to stop output.\n");
    getchar();

    error_code = SetUSB5AoImmediately(DevIndex, AOChannel_0, voltage_0);
    printf("%d AOChannel_0 is 0V\n", error_code);

    // Disable Ao software trig, clear flag
    error_code = SetUSB5ClrAoTrigger(DevIndex, AOChannel_0);
    printf("%d\n", error_code);

    // Clear FIFO
    error_code = SetUSB5ClrAoFifo(DevIndex, AOChannel_0);
    printf("%d\n", error_code);

    // Close Device and stop Ao wave output
    error_code = USB5CloseDevice(DevIndex);
    printf("%d\n", error_code);

    return 0;
}
