// @Name: di_oneshot.cpp
// @Author: Smacq
// @Copyright: Smacq
// @Date: 2025/02/08
// @Brief:
// This code focuses on the Di functions of the USB - 5000 device, performing digital input.
// It acquires 100 data points from the Di channel 10 times.
// The DI default suspended is 3.3V HIGH, and 0 - 1.2V is LOW.
// The DI channels vary depending on the module, either 2DI or 4DI or 8DI.
// The oneshot limit is 110 data points, similar to the acquisition in other functions.
// You can refer to the user manual for more information about functions or device details.
//
// Execution result when only connecting USB for USB - 5000(DI_4):
// 0
// 0
// 0
// 0
// 0
// 0
// 0
// 0
// remain_buffer: 2
// Di_channel 100 data points
//  [15 15 15 15 15 15 15 15 15 15 15 15 15 15 15 15 15 15 15 15 15 15 15 15
//  15 15 15 15 15 15 15 15 15 15 15 15 15 15 15 15 15 15 15 15 15 15 15 15
//  15 15 15 15 15 15 15 15 15 15 15 15 15 15 15 15 15 15 15 15 15 15 15 15
//  15 15 15 15 15 15 15 15 15 15 15 15 15 15 15 15 15 15 15 15 15 15 15 15
//  15 15 15 15]
// Last_remain_buffer: 11
// 0
// 0

#include <QCoreApplication>
#include <QDebug>
#include <QThread>
extern"C"{
#include "USB5000.h"
}
int main(int argc, char *argv[])
{
    QCoreApplication a(argc, argv);
    // Device index starts from zero
    int DevIndex = 0;

    // Init Di_data_array
    // Usually the array size need >= acquisition data points size.
    // The data points saved to Di_data_array arrange from opened Di channel
    // Data type is uint8, it is not modifiable.
    // Di_channel_4:0~0xF(1111) Di3~DI0, Di_channel_8:0~0xFF(1111 1111) Di7~DI0.
    unsigned char Di_data_array[1000];

    // Di channel sample mode:
    // 0 = Continuous acquisition or 1 = Oneshot, param support 0/1
    unsigned char DiSampleMode = 1;

    // DiOneShotPoints > Num, limit 110 all data point number
    unsigned int DiOneShotPoints = 110;

    // When acquisition amount is 0, just check FIFO return:
    unsigned long Zero = 0;

    // Acquisition amount of data:
    // Each opened Di channel will get this amount of data.
    // Suggest value within sampling rate 10%~20%.
    unsigned long Points = 100;

    // Di Sampling period, unit ns, steps 10ns:
    // USB - 3000 is synchronous DI, channel max rate is 10M = 100.
    // sample rate and period:
    // 1M = 1000
    // 100k = 10000
    // 10k = 100000
    // 1k = 1000000
    // 100 = 10000000
    // 10 = 100000000
    // 1 = 1000000000
    unsigned int DiSamplePeriod = 1000000;

    // Di acquisition trig source:
    // 0 = SetUSB5DiSoftTrig(), 1 = Ext_Trig_In, 2~5 = Din_0~3,
    // 6 = with Di, 8 = with DO, 9~12 = with AO~3, 255 = GlobalSoftTrig
    // Supported parameters: 0/1/2/3/4/5/6/8/9/11/12/255
    char DiTrigSource = 0;

    // Di acquisition clock source:
    // 0 = SetUSB5DiSampleRate(), 1 = Conv_In, 2 = Di, 4 = DO, 5~8 = AO~3
    // Supported parameters: 0/1/2/4/5/6/7/8
    char DiConvSource = 0;

    // Timeout unit ms:
    // Suggest Timeout>(Num*Channel_count/Samplerate)*1000
    // Avoid USB5GetDi() func timeout, if timeout return -7.
    long Timeout = 200;

    int error_code;

    // Open specific device and print return value.
    error_code = USB5OpenDevice(DevIndex);
    printf("%d\n", error_code);

    // Don't need SetUSB5ClrDiTrigger(), set limit 110 data point acquisition
    error_code = SetUSB5DiOneShotPoints(DevIndex, DiOneShotPoints);
    printf("%d\n", error_code);

    // Set Device Di sample mode
    error_code = SetUSB5DiSampleMode(DevIndex, DiSampleMode);
    printf("%d\n", error_code);

    // Set Device sample rate
    error_code = SetUSB5DiSampleRate(DevIndex, DiSamplePeriod);
    printf("%d\n", error_code);

    // Set Device clock source
    error_code = SetUSB5DiConvSource(DevIndex, DiConvSource);
    printf("%d\n", error_code);

    // Clear Di FIFO
    error_code = SetUSB5ClrDiFifo(DevIndex);
    printf("%d\n", error_code);

    // Set Device trig source
    error_code = SetUSB5DiTrigSource(DevIndex, DiTrigSource);
    printf("%d\n", error_code);

    // Enable Di software trig acquisition
    error_code = SetUSB5DiSoftTrig(DevIndex);
    printf("%d\n", error_code);

    unsigned long remain_buffer = USB5GetDi(DevIndex, Points, Di_data_array, Timeout);
    /*
    remain_buffer range 0~2000000
    USB5GetDi() return the remaining Device FIFO buffer value.
    FIFO buffer 100 data points save to Di_data_array.
    Channel_0 100 data points is Di_data_array[0:100]

    If remain_buffer increase you should run USB5GetDi() more quick, let remain_buffer steady.
    */
    printf("remain_buffer: %ld\n", remain_buffer);
    printf("Di_channel 100 data points\n");
    for (int i = 0; i < 100; ++i) {
        printf("%d ", Di_data_array[i]);
    }
    printf("\n");

    QThread::sleep(3);

    // Print and check FIFO remain 10 data point
    remain_buffer = USB5GetDi(DevIndex, Zero, Di_data_array, Timeout);
    printf("Last_remain_buffer: %ld\n", remain_buffer);

    // Clear Di FIFO
    error_code = SetUSB5ClrDiFifo(DevIndex);
    printf("%d\n", error_code);

    // Close Device
    error_code = USB5CloseDevice(DevIndex);
    printf("%d\n", error_code);

    return 0;
}
