// @Name: do_waveform_pc_infinite.cpp
// @Author: Smacq
// @Copyright: Smacq
// @Date: 2025/02/08
// @Brief:
// This code focuses on the Do functions of the USB - 5000 device, enabling two types of digital waveform output.
// It sets the Do output to HIGH/LOW, where 0V is LOW (default) and 3.3V is HIGH.
// The DO channels vary depending on the module, either Do_2 or Do_4 or Do_8.
// You can refer to the user manual for more information about functions or device details.
//
// Execution result when only connecting USB for USB - 5000:
// 0
// ...
// 0
// Do0~Do7 Flowing HIGH Interval of 1s, Please Enter to new wave.
// 0
// 0
// 0
// 0
// Do0~Do7 HiHG/LOW Interval of 1s, Please Enter to stop Do output.
// 0
// 0
// 0

#include <QCoreApplication>
#include <QDebug>
extern"C"{
#include "USB5000.h"
}
#include <QThread>
int main(int argc, char *argv[])
{
    QCoreApplication a(argc, argv);
    // Initialize the Do data array
    // The data type is uint32 and it is not modifiable.
    // Do data point 0 represents LOW, and 1 represents HIGH.
    // For example:
    // Do_channel_4: 0xF (1111) means Do3~Do0 are HIGH
    // Do_channel_8: 0xFF (1111 1111) means Do7~Do0 are HIGH
    // Do_channel_4: 0x3 (0011) means Do3~Do2 are LOW, and Do1~Do0 are HIGH
    // Do_channel_8: 0x88 (1000 1000) means Do7 and Do3 are HIGH, and other Dos are LOW
    const int firstDataPoints = 8;
    unsigned int Do_data_array_first[firstDataPoints] = {1, 2, 4, 8, 16, 32, 64, 128};

    const int secondDataPoints = 2;
    unsigned int Do_data_array_second[secondDataPoints] = {0x0, 0xFF};

    // Device index starts from zero
    int DevIndex = 0;

    // Select digital output wave channel
    // Do_channel_4: 0 (0000) ~ 0xF (1111) corresponds to Do3~Do0
    // Do_channel_8: 0 (0000 0000) ~ 0xFF (1111 1111) corresponds to Do7~Do0
    // Examples of selecting Do channels:
    // Do_0: 0x1 (0001), Do_0 and Do_1: 0x3 (0011), Do_3 and Do_7: 0x88 (1000 1000)
    unsigned int DoWaveChannel = 0xFF;

    // Do channel sample mode
    // 0 = Continuous non - loop output to the computer buffer
    // 1 = FIFO output loop determined by SetUSB5DoCycle()
    unsigned char DoSampleMode = 1;

    // Do sampling period, unit ns, steps 10ns
    // USB - 3000 is synchronous Do, and the maximum rate is 10M = 100.
    // Sampling rate and period:
    // 1M = 1000
    // 100k = 10000
    // 10k = 100000
    // 1k = 1000000
    // 100 = 10000000
    // 10 = 100000000
    // 1 = 1000000000
    unsigned int DoSampleRate = 1000000000;

    // Do acquisition trig source
    // 0 = SetUSB5AiSoftTrig(), 1 = Ext_Trig_In, 2~5 = Din_0~3,
    // 6 = with AI, 7 = with DI, 9~12 = with AO~3, 255 = GlobalSoftTrig
    // Supported parameters: 0/1/2/3/4/5/6/7/9/11/12/255
    char DoTrigSource = 0;

    // Do acquisition clock source
    // 0 = SetUSB5DoSampleRate(), 1 = Conv_In, 2 = Ai, 3 = Di, 5~8 = AO~3
    // Supported parameters: 0/1/2/3/5/6/7/8
    char DoConvSource = 0;

    // Set waveform output cycle
    // 0 = Infinite loop output
    // 1 = 1 time, 2 = 2 times, and so on.
    // Supported parameters: 0 ~ 4294967295
    unsigned int DoCycle = 0;

    int error_code;

    // Open specific device and print return value
    error_code = USB5OpenDevice(DevIndex);
    printf("%d\n", error_code);

    // Disable Do software trig, clear flag
    error_code = SetUSB5ClrDoTrigger(DevIndex);
    printf("%d\n", error_code);

    // Clear FIFO
    error_code = SetUSB5ClrDoFifo(DevIndex);
    printf("%d\n", error_code);

    // Set Device Do sample mode
    error_code = SetUSB5DoSampleMode(DevIndex, DoSampleMode);
    printf("%d\n", error_code);

    // Set Device sample rate
    error_code = SetUSB5DoSampleRate(DevIndex, DoSampleRate);
    printf("%d\n", error_code);

    // Set digital output wave channel
    error_code = SetUSB5DoWaveCtrl(DevIndex, DoWaveChannel);
    printf("%d\n", error_code);

    // Set digital output cycle infinite loop
    error_code = SetUSB5DoCycle(DevIndex, DoCycle);
    printf("%d\n", error_code);

    // Set Device trig source
    error_code = SetUSB5DoTrigSource(DevIndex, DoTrigSource);
    printf("%d\n", error_code);

    // Set Device clock source
    error_code = SetUSB5DoConvSource(DevIndex, DoConvSource);
    printf("%d\n", error_code);

    // Download array data to Do FIFO
    error_code = SetUSB5DoDataFifo(DevIndex, Do_data_array_first, firstDataPoints);
    printf("%d\n", error_code);

    // Enable Do software trig start DO output wave
    error_code = SetUSB5DoSoftTrig(DevIndex);
    printf("%d\n", error_code);

    // Wait for user input
    printf("Do0~Do7 Flowing HIGH Interval of 1s, Please Enter to new wave.\n");
    getchar();

    // Disable Do software trig, clear flag
    error_code = SetUSB5ClrDoTrigger(DevIndex);
    printf("%d\n", error_code);

    // Clear FIFO
    error_code = SetUSB5ClrDoFifo(DevIndex);
    printf("%d\n", error_code);

    // Update new wave
    // Download array data to Do FIFO
    error_code = SetUSB5DoDataFifo(DevIndex, Do_data_array_second, secondDataPoints);
    printf("%d\n", error_code);

    // Enable Do software trig start DO output wave
    error_code = SetUSB5DoSoftTrig(DevIndex);
    printf("%d\n", error_code);

    // Wait for user input
    printf("Do0~Do7 HiHG/LOW Interval of 1s, Please Enter to stop Do output.\n");
    getchar();

    // Disable Do software trig, clear flag
    error_code = SetUSB5ClrDoTrigger(DevIndex);
    printf("%d\n", error_code);

    // Clear FIFO
    error_code = SetUSB5ClrDoFifo(DevIndex);
    printf("%d\n", error_code);

    // Close Device and stop Do wave output
    error_code = USB5CloseDevice(DevIndex);
    printf("%d\n", error_code);

    return 0;
}
